@@ -9,6 +9,8 @@ from re import match
 import sh
 import shutil
 import fnmatch
+import urllib.request
+from urllib.request import urlretrieve
 from os import listdir, unlink, environ, mkdir, curdir, walk
 from sys import stdout
 import time
@@ -17,22 +19,14 @@ try:
 except ImportError:
     from urllib.parse import urlparse
 from pythonforandroid.logger import (logger, info, warning, debug, shprint, info_main)
-from pythonforandroid.util import (urlretrieve, current_directory, ensure_dir,
+from pythonforandroid.util import (current_directory, ensure_dir,
                                    BuildInterruptingException)
+from pythonforandroid.util import load_source as import_recipe
 
 
-def import_recipe(module, filename):
-    # Python 3.5+
-    import importlib.util
-    if hasattr(importlib.util, 'module_from_spec'):
-        spec = importlib.util.spec_from_file_location(module, filename)
-        mod = importlib.util.module_from_spec(spec)
-        spec.loader.exec_module(mod)
-        return mod
-    else:
-        # Python 3.3 and 3.4:
-        from importlib.machinery import SourceFileLoader
-        return SourceFileLoader(module, filename).load_module()
+url_opener = urllib.request.build_opener()
+url_orig_headers = url_opener.addheaders
+urllib.request.install_opener(url_opener)
 
 
 class RecipeMeta(type):
@@ -71,6 +65,18 @@ class Recipe(with_metaclass(RecipeMeta)):
     finished correctly.
     '''
 
+    sha512sum = None
+    '''The sha512sum of the source from the :attr:`url`. Non-essential, but
+    you should try to include this, it is used to check that the download
+    finished correctly.
+    '''
+
+    blake2bsum = None
+    '''The blake2bsum of the source from the :attr:`url`. Non-essential, but
+    you should try to include this, it is used to check that the download
+    finished correctly.
+    '''
+
     depends = []
     '''A list containing the names of any recipes that this recipe depends on.
     '''
@@ -129,24 +135,9 @@ class Recipe(with_metaclass(RecipeMeta)):
         starting from NDK r18 the `gnustl_shared` lib has been deprecated.
     '''
 
-    stl_lib_source = '{ctx.ndk_dir}/sources/cxx-stl/llvm-libc++'
-    '''
-    The source directory of the selected stl lib, defined in property
-    `stl_lib_name`
-    '''
-
-    @property
-    def stl_include_dir(self):
-        return join(self.stl_lib_source.format(ctx=self.ctx), 'include')
-
-    def get_stl_lib_dir(self, arch):
-        return join(
-            self.stl_lib_source.format(ctx=self.ctx), 'libs', arch.arch
-        )
-
     def get_stl_library(self, arch):
         return join(
-            self.get_stl_lib_dir(arch),
+            arch.ndk_lib_dir,
             'lib{name}.so'.format(name=self.stl_lib_name),
         )
 
@@ -203,26 +194,35 @@ class Recipe(with_metaclass(RecipeMeta)):
 
             # Download item with multiple attempts (for bad connections):
             attempts = 0
+            seconds = 1
             while True:
                 try:
+                    # jqueryui.com returns a 403 w/ the default user agent
+                    # Mozilla/5.0 doesnt handle redirection for liblzma
+                    url_opener.addheaders = [('User-agent', 'Wget/1.0')]
                     urlretrieve(url, target, report_hook)
-                except OSError:
+                except OSError as e:
                     attempts += 1
                     if attempts >= 5:
                         raise
-                    stdout.write('Download failed retrying in a second...')
-                    time.sleep(1)
+                    stdout.write('Download failed: {}; retrying in {} second(s)...'.format(e, seconds))
+                    time.sleep(seconds)
+                    seconds *= 2
                     continue
+                finally:
+                    url_opener.addheaders = url_orig_headers
                 break
             return target
         elif parsed_url.scheme in ('git', 'git+file', 'git+ssh', 'git+http', 'git+https'):
             if isdir(target):
                 with current_directory(target):
-                    shprint(sh.git, 'fetch', '--tags')
+                    shprint(sh.git, 'fetch', '--tags', '--recurse-submodules')
                     if self.version:
                         shprint(sh.git, 'checkout', self.version)
-                    shprint(sh.git, 'pull')
-                    shprint(sh.git, 'pull', '--recurse-submodules')
+                    branch = sh.git('branch', '--show-current')
+                    if branch:
+                        shprint(sh.git, 'pull')
+                        shprint(sh.git, 'pull', '--recurse-submodules')
                     shprint(sh.git, 'submodule', 'update', '--recursive')
             else:
                 if url.startswith('git+'):
@@ -352,16 +352,19 @@ class Recipe(with_metaclass(RecipeMeta)):
             return
 
         url = self.versioned_url
-        ma = match(u'^(.+)#md5=([0-9a-f]{32})$', url)
-        if ma:                  # fragmented URL?
-            if self.md5sum:
-                raise ValueError(
-                    ('Received md5sum from both the {} recipe '
-                     'and its url').format(self.name))
-            url = ma.group(1)
-            expected_md5 = ma.group(2)
-        else:
-            expected_md5 = self.md5sum
+        expected_digests = {}
+        for alg in set(hashlib.algorithms_guaranteed) | set(('md5', 'sha512', 'blake2b')):
+            expected_digest = getattr(self, alg + 'sum') if hasattr(self, alg + 'sum') else None
+            ma = match(u'^(.+)#' + alg + u'=([0-9a-f]{32,})$', url)
+            if ma:                # fragmented URL?
+                if expected_digest:
+                    raise ValueError(
+                        ('Received {}sum from both the {} recipe '
+                         'and its url').format(alg, self.name))
+                url = ma.group(1)
+                expected_digest = ma.group(2)
+            if expected_digest:
+                expected_digests[alg] = expected_digest
 
         shprint(sh.mkdir, '-p', join(self.ctx.packages_path, self.name))
 
@@ -373,16 +376,17 @@ class Recipe(with_metaclass(RecipeMeta)):
             if exists(filename) and isfile(filename):
                 if not exists(marker_filename):
                     shprint(sh.rm, filename)
-                elif expected_md5:
-                    current_md5 = md5sum(filename)
-                    if current_md5 != expected_md5:
-                        debug('* Generated md5sum: {}'.format(current_md5))
-                        debug('* Expected md5sum: {}'.format(expected_md5))
-                        raise ValueError(
-                            ('Generated md5sum does not match expected md5sum '
-                             'for {} recipe').format(self.name))
-                    do_download = False
                 else:
+                    for alg, expected_digest in expected_digests.items():
+                        current_digest = algsum(alg, filename)
+                        if current_digest != expected_digest:
+                            debug('* Generated {}sum: {}'.format(alg,
+                                                                 current_digest))
+                            debug('* Expected {}sum: {}'.format(alg,
+                                                                expected_digest))
+                            raise ValueError(
+                                ('Generated {0}sum does not match expected {0}sum '
+                                 'for {1} recipe').format(alg, self.name))
                     do_download = False
 
             # If we got this far, we will download
@@ -393,15 +397,17 @@ class Recipe(with_metaclass(RecipeMeta)):
                 self.download_file(self.versioned_url, filename)
                 shprint(sh.touch, marker_filename)
 
-                if exists(filename) and isfile(filename) and expected_md5:
-                    current_md5 = md5sum(filename)
-                    if expected_md5 is not None:
-                        if current_md5 != expected_md5:
-                            debug('* Generated md5sum: {}'.format(current_md5))
-                            debug('* Expected md5sum: {}'.format(expected_md5))
+                if exists(filename) and isfile(filename):
+                    for alg, expected_digest in expected_digests.items():
+                        current_digest = algsum(alg, filename)
+                        if current_digest != expected_digest:
+                            debug('* Generated {}sum: {}'.format(alg,
+                                                                 current_digest))
+                            debug('* Expected {}sum: {}'.format(alg,
+                                                                expected_digest))
                             raise ValueError(
-                                ('Generated md5sum does not match expected md5sum '
-                                 'for {} recipe').format(self.name))
+                                ('Generated {0}sum does not match expected {0}sum '
+                                 'for {1} recipe').format(alg, self.name))
             else:
                 info('{} download already cached, skipping'.format(self.name))
 
@@ -429,7 +435,7 @@ class Recipe(with_metaclass(RecipeMeta)):
 
         filename = shprint(
             sh.basename, self.versioned_url).stdout[:-1].decode('utf-8')
-        ma = match(u'^(.+)#md5=([0-9a-f]{32})$', filename)
+        ma = match(u'^(.+)#[a-z0-9_]{3,}=([0-9a-f]{32,})$', filename)
         if ma:                  # fragmented URL?
             filename = ma.group(1)
 
@@ -489,14 +495,14 @@ class Recipe(with_metaclass(RecipeMeta)):
 
         if self.need_stl_shared:
             env['CPPFLAGS'] = env.get('CPPFLAGS', '')
-            env['CPPFLAGS'] += ' -I{}'.format(self.stl_include_dir)
+            env['CPPFLAGS'] += ' -I{}'.format(self.ctx.ndk.libcxx_include_dir)
 
             env['CXXFLAGS'] = env['CFLAGS'] + ' -frtti -fexceptions'
 
             if with_flags_in_cc:
                 env['CXX'] += ' -frtti -fexceptions'
 
-            env['LDFLAGS'] += ' -L{}'.format(self.get_stl_lib_dir(arch))
+            env['LDFLAGS'] += ' -L{}'.format(arch.ndk_lib_dir)
             env['LIBS'] = env.get('LIBS', '') + " -l{}".format(
                 self.stl_lib_name
             )
@@ -707,7 +713,7 @@ class Recipe(with_metaclass(RecipeMeta)):
             if recipe_file is not None:
                 break
 
-        if not recipe_file:
+        else:
             raise ValueError('Recipe does not exist: {}'.format(name))
 
         mod = import_recipe('pythonforandroid.recipes.{}'.format(name), recipe_file)
@@ -767,9 +773,7 @@ class BootstrapNDKRecipe(Recipe):
         env['PYTHON_INCLUDE_ROOT'] = self.ctx.python_recipe.include_root(arch.arch)
         env['PYTHON_LINK_ROOT'] = self.ctx.python_recipe.link_root(arch.arch)
         env['EXTRA_LDLIBS'] = ' -lpython{}'.format(
-            self.ctx.python_recipe.major_minor_version_string)
-        if 'python3' in self.ctx.python_recipe.name:
-            env['EXTRA_LDLIBS'] += 'm'
+            self.ctx.python_recipe.link_version)
         return env
 
 
@@ -799,7 +803,7 @@ class NDKRecipe(Recipe):
         env = self.get_recipe_env(arch)
         with current_directory(self.get_build_dir(arch.arch)):
             shprint(
-                sh.ndk_build,
+                sh.Command(join(self.ctx.ndk_dir, "ndk-build")),
                 'V=1',
                 'NDK_DEBUG=' + ("1" if self.ctx.build_as_debuggable else "0"),
                 'APP_PLATFORM=android-' + str(self.ctx.ndk_api),
@@ -906,16 +910,13 @@ class PythonRecipe(Recipe):
         env['LANG'] = "en_GB.UTF-8"
 
         if not self.call_hostpython_via_targetpython:
-            python_name = self.ctx.python_recipe.name
             env['CFLAGS'] += ' -I{}'.format(
                 self.ctx.python_recipe.include_root(arch.arch)
             )
             env['LDFLAGS'] += ' -L{} -lpython{}'.format(
                 self.ctx.python_recipe.link_root(arch.arch),
-                self.ctx.python_recipe.major_minor_version_string,
+                self.ctx.python_recipe.link_version,
             )
-            if python_name == 'python3':
-                env['LDFLAGS'] += 'm'
 
             hppath = []
             hppath.append(join(dirname(self.hostpython_location), 'Lib'))
@@ -933,7 +934,7 @@ class PythonRecipe(Recipe):
 
     def should_build(self, arch):
         name = self.folder_name
-        if self.ctx.has_package(name):
+        if self.ctx.has_package(name, arch):
             info('Python package already exists in site-packages')
             return False
         info('{} apparently isn\'t already in site-packages'.format(name))
@@ -956,11 +957,11 @@ class PythonRecipe(Recipe):
 
         info('Installing {} into site-packages'.format(self.name))
 
+        hostpython = sh.Command(self.hostpython_location)
+        hpenv = env.copy()
         with current_directory(self.get_build_dir(arch.arch)):
-            hostpython = sh.Command(self.hostpython_location)
-            hpenv = env.copy()
             shprint(hostpython, 'setup.py', 'install', '-O2',
-                    '--root={}'.format(self.ctx.get_python_install_dir()),
+                    '--root={}'.format(self.ctx.get_python_install_dir(arch.arch)),
                     '--install-lib=.',
                     _env=hpenv, *self.setup_extra_args)
 
@@ -999,8 +1000,8 @@ class CompiledComponentsPythonRecipe(PythonRecipe):
         info('Building compiled components in {}'.format(self.name))
 
         env = self.get_recipe_env(arch)
+        hostpython = sh.Command(self.hostpython_location)
         with current_directory(self.get_build_dir(arch.arch)):
-            hostpython = sh.Command(self.hostpython_location)
             if self.install_in_hostpython:
                 shprint(hostpython, 'setup.py', 'clean', '--all', _env=env)
             shprint(hostpython, 'setup.py', self.build_cmd, '-v',
@@ -1072,7 +1073,7 @@ class CythonRecipe(PythonRecipe):
                 info('First build appeared to complete correctly, skipping manual'
                      'cythonising.')
 
-            if not self.ctx.build_as_debuggable:
+            if not self.ctx.with_debug_symbols:
                 self.strip_object_files(arch, env)
 
     def strip_object_files(self, arch, env, build_dir=None):
@@ -1102,7 +1103,8 @@ class CythonRecipe(PythonRecipe):
         python_command = sh.Command("python{}".format(
             self.ctx.python_recipe.major_minor_version_string.split(".")[0]
         ))
-        shprint(python_command, "-m", "Cython.Build.Cythonize",
+        shprint(python_command, "-c"
+                "import sys; from Cython.Compiler.Main import setuptools_main; sys.exit(setuptools_main());",
                 filename, *self.cython_args, _env=cyenv)
 
     def cythonize_build(self, env, build_dir="."):
@@ -1125,7 +1127,6 @@ class CythonRecipe(PythonRecipe):
         env['LDSHARED'] = env['CC'] + ' -shared'
         # shprint(sh.whereis, env['LDSHARED'], _env=env)
         env['LIBLINK'] = 'NOTNONE'
-        env['NDKPLATFORM'] = self.ctx.ndk_platform
         if self.ctx.copy_libs:
             env['COPYLIBS'] = '1'
 
@@ -1186,10 +1187,10 @@ class TargetPythonRecipe(Recipe):
             shprint(sh.mv, filen, join(file_dirname, parts[0] + '.so'))
 
 
-def md5sum(filen):
-    '''Calculate the md5sum of a file.
+def algsum(alg, filen):
+    '''Calculate the digest of a file.
     '''
     with open(filen, 'rb') as fileh:
-        md5 = hashlib.md5(fileh.read())
+        digest = getattr(hashlib, alg)(fileh.read())
 
-    return md5.hexdigest()
+    return digest.hexdigest()
