@@ -1,17 +1,18 @@
 import sh
+import os
 
 from multiprocessing import cpu_count
 from pathlib import Path
 from os.path import join
 
 from pythonforandroid.logger import shprint
-from pythonforandroid.patching import is_version_lt
 from pythonforandroid.recipe import Recipe
 from pythonforandroid.util import (
     BuildInterruptingException,
     current_directory,
     ensure_dir,
 )
+from pythonforandroid.prerequisites import OpenSSLPrerequisite
 
 HOSTPYTHON_VERSION_UNSET_MESSAGE = (
     'The hostpython recipe must have set version'
@@ -34,7 +35,7 @@ class HostPython3Recipe(Recipe):
         :class:`~pythonforandroid.python.HostPythonRecipe`
     '''
 
-    version = '3.8.1'
+    version = '3.9.9'
     name = 'hostpython3'
 
     build_subdir = 'native-build'
@@ -45,9 +46,7 @@ class HostPython3Recipe(Recipe):
     '''The default url to download our host python recipe. This url will
     change depending on the python version set in attribute :attr:`version`.'''
 
-    patches = (
-        ('patches/pyconfig_detection.patch', is_version_lt("3.8.3")),
-    )
+    patches = ['patches/pyconfig_detection.patch']
 
     @property
     def _exe_name(self):
@@ -63,6 +62,17 @@ class HostPython3Recipe(Recipe):
         '''Returns the full path of the hostpython executable.'''
         return join(self.get_path_to_python(), self._exe_name)
 
+    def get_recipe_env(self, arch=None):
+        env = os.environ.copy()
+        openssl_prereq = OpenSSLPrerequisite()
+        if env.get("PKG_CONFIG_PATH", ""):
+            env["PKG_CONFIG_PATH"] = os.pathsep.join(
+                openssl_prereq.pkg_config_location, env["PKG_CONFIG_PATH"]
+            )
+        else:
+            env["PKG_CONFIG_PATH"] = openssl_prereq.pkg_config_location
+        return env
+
     def should_build(self, arch):
         if Path(self.python_exe).exists():
             # no need to build, but we must set hostpython for our Context
@@ -86,6 +96,8 @@ class HostPython3Recipe(Recipe):
         return join(self.get_build_dir(), self.build_subdir)
 
     def build_arch(self, arch):
+        env = self.get_recipe_env(arch)
+
         recipe_build_dir = self.get_build_dir(arch.arch)
 
         # Create a subdirectory to actually perform the build
@@ -95,7 +107,7 @@ class HostPython3Recipe(Recipe):
         # Configure the build
         with current_directory(build_dir):
             if not Path('config.status').exists():
-                shprint(sh.Command(join(recipe_build_dir, 'configure')))
+                shprint(sh.Command(join(recipe_build_dir, 'configure')), _env=env)
 
         with current_directory(recipe_build_dir):
             # Create the Setup file. This copying from Setup.dist is
@@ -113,7 +125,7 @@ class HostPython3Recipe(Recipe):
                         SETUP_DIST_NOT_FIND_MESSAGE
                     )
 
-            shprint(sh.make, '-j', str(cpu_count()), '-C', build_dir)
+            shprint(sh.make, '-j', str(cpu_count()), '-C', build_dir, _env=env)
 
             # make a copy of the python executable giving it the name we want,
             # because we got different python's executable names depending on
