@@ -25,7 +25,7 @@ class ProtobufCppRecipe(CppCompiledComponentsPythonRecipe):
     protoc_dir = None
 
     def prebuild_arch(self, arch):
-        super(ProtobufCppRecipe, self).prebuild_arch(arch)
+        super().prebuild_arch(arch)
 
         patch_mark = join(self.get_build_dir(arch.arch), '.protobuf-patched')
         if self.ctx.python_recipe.name == 'python3' and not exists(patch_mark):
@@ -115,7 +115,7 @@ class ProtobufCppRecipe(CppCompiledComponentsPythonRecipe):
 
             hpenv = env.copy()
             shprint(hostpython, 'setup.py', 'install', '-O2',
-                    '--root={}'.format(self.ctx.get_python_install_dir()),
+                    '--root={}'.format(self.ctx.get_python_install_dir(arch.arch)),
                     '--install-lib=.',
                     _env=hpenv, *self.setup_extra_args)
 
@@ -124,12 +124,12 @@ class ProtobufCppRecipe(CppCompiledComponentsPythonRecipe):
         #   - https://stackoverflow.com/questions/13862562/
         #   google-protocol-buffers-not-found-when-trying-to-freeze-python-app
         open(
-            join(self.ctx.get_site_packages_dir(), 'google', '__init__.py'),
+            join(self.ctx.get_site_packages_dir(arch), 'google', '__init__.py'),
             'a',
         ).close()
 
     def get_recipe_env(self, arch):
-        env = super(ProtobufCppRecipe, self).get_recipe_env(arch)
+        env = super().get_recipe_env(arch)
         if self.protoc_dir is not None:
             # we need protoc with binary for host platform
             env['PROTOC'] = join(self.protoc_dir, 'bin', 'protoc')
